from Crypto.Random import random
import requests


class Client:
    RequestFailed = type('RequestFailed', (Exception,), {})

    class dto_class(dict):
        def __getattr__(self, name):
            if name not in self:
                raise AttributeError(name)
            return self[name]

    def __init__(self, host, base_url='/', scheme="http"):
        self.host = host
        self.base_url = base_url
        self.scheme = scheme

    def request_confirmation(self, display_name, phonenumber, template, expires=None):
        """Send a request to verify to the given `phonenumber`.

        Args:
            display_name: the name to be displayed to the user; max
                of 11 character.
            phonenumber: a valid ITU-T E.164 phonenumber.
            template: a message template containing a ``{token}``
                variable. No more than 64 characters.
            expires: an optional unsigned long integer specifying when
                the request expires, in milliseconds since the UNIX epoch.
        """
        url = self._make_url('/phonenumbers/confirm/')
        if "{token}" not in template:
            raise ValueError("template string must contain {token}")

        if len(display_name) > 11:
            raise ValueError("display_name may be at most 11 characters.")

        token = random.randint(100000, 999999)
        params = {
            'message': template.format(token=token),
            'token': str(token),
            'phonenumber': phonenumber,
            'display_name': display_name,
        }
        if expires is not None:
            params['expires'] = expires

        response = requests.post(url, json=params)
        if response.status_code >= 400:
            raise RequestFailed(url, response.status_code, response.get_data())

        return self.dto_class(**response.json())

    def _make_url(self, path):
        return "%s://%s%s/%s" % (self.scheme, self.host,
            self.base_url, path.lstrip('/'))



import os

from werkzeug.routing import Map
from werkzeug.routing import Rule
from werkzeug.debug import DebuggedApplication
from werkzeug.serving import run_simple
from werkzeug.wrappers import Response
from wizards.wsgi import BaseApplication
import ioc
import yaml

from pvs.controllers import ConfirmationRequestController

urls = Map([
    Rule(
        '/phonenumbers/confirm/',
        endpoint='phonenumber:confirm:list'
    ),
    Rule(
        '/phonenumbers/confirm/<uuid:request_id>',
        endpoint='phonenumber:confirm:detail'
    ),
])

handlers = {
    "phonenumber:confirm:list": ConfirmationRequestController.as_view()
}

PVS_STORAGE_IOC = 'etc/pvs-storage.ioc'
PVS_SERVICES_IOC = 'etc/pvs-services.ioc'
PVS_READMODEL_IOC = 'etc/pvs-readmodel.ioc'
PVS_CREDENTIALS_IOC = 'etc/pvs-credentials.ioc'
ioc.provide('APIController.response_class', Response)

with open(PVS_SERVICES_IOC) as f:
    ioc.load(yaml.safe_load(f.read()))
with open(PVS_STORAGE_IOC) as f:
    ioc.load(yaml.safe_load(f.read()))

with open(PVS_CREDENTIALS_IOC) as f:
    ioc.load(yaml.safe_load(f.read()))

class Application(BaseApplication):
    pass


application = Application(urls,
    handlers=handlers,
    debug=__name__=='__main__' or os.getenv('PVS_DEBUG'),
    content_types=["application/json"]
)

if __name__ == '__main__':
    application = DebuggedApplication(application)
    run_simple('0.0.0.0', 8000, application, use_reloader=True, use_debugger=True)



#!/usr/bin/env python3
from distutils.core import setup


setup(
    name='service-phonenumbers',
    version='1.0.0',
    description='Phonenumber Microservice client',
    author='Cochise Ruhulessin',
    author_email='cochiseruhulessin@gmail.com',
    url='https://www.wizardsofindustry.net',
    install_requires=[
        'requests',
        'PyCrypto'
    ],
    packages=[
        'pvs'
    ]
)
